
'use strict';

let clone = require('clone');
let co = require('co');
let empty = require('is-empty');
let fs = require('mz/fs');
let glob = require('globby');
let merge = require('merge-array');
let path = require('path');
let resolve = require('resolve').sync;

// single export
exports = module.exports = co.wrap(load);
exports.sync = loadSync;

/**
 * Loads a mako configuration file.
 *
 * @param {String} file        The absolute pathname to the config file to load.
 * @param {Array} [overrides]  User-supplied entries. (overrides config)
 * @return {Object}
 */
function* load(file, overrides) {
  let config = JSON.parse(yield fs.readFile(file, 'utf8'));
  let dir = path.dirname(file);
  let env = config[process.env.NODE_ENV || 'development'];
  let root = config.root ? path.resolve(dir, config.root) : null;

  if (env) {
    if (env.entries) merge(config.entries, env.entries);
    if (env.plugins) merge(config.plugins, env.plugins);
  }

  return {
    entries: yield normalizeEntries(overrides, config.entries, dir, root),
    plugins: normalizePlugins(config.plugins, dir)
  };
}

/**
 * Synchronous version of load.
 *
 * @param {String} file        The absolute pathname to the config file to load.
 * @param {Array} [overrides]  User-supplied entries. (overrides config)
 * @return {Object}
 */
function loadSync(file, overrides) {
  let config = JSON.parse(fs.readFileSync(file, 'utf8'));
  let dir = path.dirname(file);
  let env = config[process.env.NODE_ENV || 'development'];
  let root = config.root ? path.resolve(dir, config.root) : null;

  if (env) {
    if (env.entries) merge(config.entries, env.entries);
    if (env.plugins) merge(config.plugins, env.plugins);
  }

  return {
    entries: normalizeEntriesSync(overrides, config.entries, dir, root),
    plugins: normalizePlugins(config.plugins, dir)
  };
}

/**
 * Normalizes the list of entries. By default, the config file will contain a
 * complete list of entries. If the user passes a list of entries via the CLI,
 * that list will be used instead.
 *
 * It will return a flattened array of absolute file paths, with any globs
 * already expanded.
 *
 * @param {Array} input    The input list from the user. (if specified)
 * @param {Array} config   The list specified in the config file.
 * @param {String} dir     The location of the config file.
 * @param {String} [root]  An alternate root path.
 * @return {Array}
 */
function* normalizeEntries(input, config, dir, root) {
  let list = !empty(input) ? input : config;
  let files = yield glob(list, { cwd: dir, realpath: true });
  return files.map(normalizeEntry(root));
}

/**
 * Synchronous version of normalizeEntries.
 *
 * @param {Array} input    The input list from the user. (if specified)
 * @param {Array} config   The list specified in the config file.
 * @param {String} dir     The location of the config file.
 * @param {String} [root]  An alternate root path.
 * @return {Array}
 */
function normalizeEntriesSync(input, config, dir, root) {
  let list = !empty(input) ? input : config;
  let files = glob.sync(list, { cwd: dir, realpath: true });
  return files.map(normalizeEntry(root));
}

/**
 * Takes a given file and normalizes it into compatible params for the
 * `File` constructor.
 *
 * @param {String} [root]  An alternate root path.
 * @return {Function}
 */
function normalizeEntry(root) {
  return function (file) {
    return root
      ? { base: root, path: file }
      : file;
  };
}

/**
 * Normalizes the list of plugins. For each plugin specified in the config,
 * it will initialize. (with arguments if supplied) The return value is a flat
 * list of plugin functions.
 *
 * When a plugin is specified only as a string, it will be initialized without
 * arguments. (basically accepting the defaults)
 *
 * When a plugin is specified as an array, the first item is the plugin name,
 * all other items will be forwarded as arguments.
 *
 * @param {Array} config  The list of plugins specified in the config.
 * @param {String} root   The location of the config file.
 * @return {Array}
 */
function normalizePlugins(config, root) {
  if (!config) return [];
  return config
    .map(plugin => {
      if (typeof plugin === 'string') return [ plugin ];
      return clone(plugin);
    })
    .map(plugin => {
      let fn = require(resolve(plugin[0], { basedir: root }));
      return fn.apply(null, plugin.slice(1));
    });
}
